@@ -32,15 +32,13 @@ from ml_metadata.metadata_store import metadata_store
 # https://github.com/google/ml-metadata/issues/25
 # https://github.com/google/ml-metadata/pull/35
 
-from kale.common import utils, podutils
+from kale.common import utils, podutils, workflowutils, k8sutils, kfputils
 
 
 DEFAULT_METADATA_GRPC_SERVICE_SERVICE_HOST = ("metadata-grpc-service.kubeflow"
                                               ".svc")
 DEFAULT_METADATA_GRPC_SERVICE_SERVICE_PORT = 8080
 
-ARGO_WORKFLOW_LABEL_KEY = "workflows.argoproj.io/workflow"
-
 RUN_CONTEXT_TYPE_NAME = "KfpRun"
 KFP_EXECUTION_TYPE_NAME_PREFIX = "components."
 
@@ -84,6 +82,7 @@ mlmd_instance = None
 
 class MLMetadata(object):
     """Contains all context for a Kale step's ML-Metadata representation."""
+
     def __init__(self):
         log.info("%s Initializing MLMD context... %s", "-" * 10, "-" * 10)
         log.info("Connecting to MLMD...")
@@ -102,12 +101,12 @@ class MLMetadata(object):
         log.info("Successfully retrieved pod")
         log.info("Getting workflow name from pod...")
         self.workflow_name = self.pod.metadata.labels.get(
-            ARGO_WORKFLOW_LABEL_KEY)
+            workflowutils.ARGO_WORKFLOW_LABEL_KEY)
         log.info("Successfully retrieved workflow name: %s",
                  self.workflow_name)
         log.info("Getting workflow...")
-        self.workflow = podutils.get_workflow(self.workflow_name,
-                                              self.pod_namespace)
+        self.workflow = workflowutils.get_workflow(self.workflow_name,
+                                                   self.pod_namespace)
         log.info("Successfully retrieved workflow")
 
         workflow_labels = self.workflow["metadata"].get("labels", {})
@@ -277,7 +276,8 @@ class MLMetadata(object):
         return context
 
     def _get_or_create_run_context(self):
-        run_id = metadata_store_pb2.Value(string_value=self.run_uuid)
+        run_id = metadata_store_pb2.Value(
+            string_value=kfputils.format_kfp_run_id_uri(self.run_uuid))
         workflow_name = metadata_store_pb2.Value(
             string_value=self.workflow_name)
         pipeline_name = metadata_store_pb2.Value(
@@ -295,7 +295,8 @@ class MLMetadata(object):
             property_types=property_types, properties=properties)
 
     def _create_execution_in_run_context(self):
-        run_id = metadata_store_pb2.Value(string_value=self.run_uuid)
+        run_id = metadata_store_pb2.Value(
+            string_value=kfputils.format_kfp_run_id_uri(self.run_uuid))
         pipeline_name = metadata_store_pb2.Value(
             string_value=self.pipeline_name)
         component_id = metadata_store_pb2.Value(string_value=self.component_id)
@@ -403,7 +404,11 @@ class MLMetadata(object):
         task_info = rok_client.version_info(bucket, obj, version)
         members = int(task_info["group_member_count"])
         url = task_info["rok_url"]
-        uri = "/rok/buckets/%s/files/%s/versions/%s" % (bucket, obj, version)
+        uri = ("/rok/buckets/%s/files/%s/versions/%s?ns=%s"
+               % (utils.encode_url_component(bucket),
+                  utils.encode_url_component(obj),
+                  utils.encode_url_component(version),
+                  utils.encode_url_component(self.pod_namespace)))
         hash_value = task_info["hash"]
 
         property_types = ROK_SNAPSHOT_ARTIFACT_PROPERTIES
@@ -441,7 +446,7 @@ class MLMetadata(object):
         # https://github.com/kubeflow/pipelines/pull/2852
         # https://github.com/kubeflow/pipelines/pull/3485#issuecomment-612722767
         custom_properties["run_id"] = metadata_store_pb2.Value(
-            string_value=self.run_uuid)
+            string_value=kfputils.format_kfp_run_id_uri(self.run_uuid))
 
         return self._create_artifact_with_type(uri,
                                                ROK_SNAPSHOT_ARTIFACT_TYPE_NAME,
@@ -470,7 +475,7 @@ class MLMetadata(object):
 
             output_artifact_ids = []
             annotation = METADATA_OUTPUT_ARTIFACT_IDS_ANNOTATION_KEY
-            k8s_client = podutils._get_k8s_v1_client()
+            k8s_client = k8sutils.get_v1_client()
             for name in pod_names:
                 pod = k8s_client.read_namespaced_pod(name, self.pod_namespace)
                 ids = json.loads(pod.metadata.annotations.get(annotation,
@@ -481,15 +486,11 @@ class MLMetadata(object):
                     in self.store.get_artifacts_by_id(output_artifact_ids)
                     if a.type_id == rok_snapshot_type_id]
 
-        # Search in workflow.Status.Nodes for step's parents
-        nodes = self.workflow["status"]["nodes"]
-
-        # Find all nodes that are direct ancestors
-        parents = []
-        for name, node in nodes.items():
-            if (self.pod_name in node.get("children", [])
-                    and name != self.workflow_name):
-                parents.append(name)
+        log.info("Searching for RokSnapshot artifacts to link as inputs...")
+        # Find all pods that are direct ancestors
+        log.info("Searching for Pod-type direct ancestors of step...")
+        parents = workflowutils.find_pod_parents(self.pod_name, self.workflow)
+        log.info("Found parent pods: %s", parents)
 
         rok_artifacts = _get_output_rok_artifacts(parents)
 
