@@ -13,10 +13,9 @@
 #  limitations under the License.
 
 import os
-import kubernetes
 from kubernetes.client.rest import ApiException
 
-from kale.common import podutils
+from kale.common import podutils, k8sutils
 from kale.rpc.errors import RPCNotFoundError, RPCUnhandledError
 
 KATIB_PARAMETER_NAMES = ("objective", "algorithm", "parallelTrialCount",
@@ -26,7 +25,7 @@ KATIB_DEFAULTS = {"parallelTrialCount": 3, "maxTrialCount": 12,
 KATIB_EXPERIMENT_STATUS = ["Failed", "Succeeded", "Restarting", "Running",
                            "Created"]
 
-KATIB_TRIAL_IMAGE = "gcr.io/arrikto/katib-kfp-trial:7a304af-feaafdd"
+KATIB_TRIAL_IMAGE = "gcr.io/arrikto/katib-kfp-trial:2d0f237-9f74f267"
 
 RAW_TEMPLATE = """\
 apiVersion: batch/v1
@@ -62,19 +61,6 @@ spec:
 """
 
 
-def _get_k8s_co_client(trans_id):
-    try:
-        kubernetes.config.load_incluster_config()
-    except Exception:  # Not in a notebook server
-        try:
-            kubernetes.config.load_kube_config()
-        except Exception:
-            raise RPCUnhandledError(details="Could not load Kubernetes config",
-                                    trans_id=trans_id)
-
-    return kubernetes.client.CustomObjectsApi()
-
-
 def _define_katib_experiment(name, katib_spec, trial_parameters):
     """Define Katib experiment."""
     katib_experiment = {"apiVersion": "kubeflow.org/v1alpha3",
@@ -95,7 +81,7 @@ def _define_katib_experiment(name, katib_spec, trial_parameters):
 
 def _launch_katib_experiment(request, katib_experiment, namespace):
     """Launch Katib experiment."""
-    k8s_co_client = _get_k8s_co_client(request.trans_id)
+    k8s_co_client = k8sutils.get_co_client()
 
     co_group = "kubeflow.org"
     co_version = "v1alpha3"
@@ -138,6 +124,14 @@ def _sanitize_katib_spec(request, katib_spec):
                                 KATIB_DEFAULTS, "Katib")
 
 
+def _construct_experiment_return_base(experiment, namespace):
+    return {"name": experiment["metadata"]["name"],
+            "namespace": namespace,
+            "status": None,
+            "trials": 0,
+            "maxTrialCount": experiment["spec"]["maxTrialCount"]}
+
+
 def create_katib_experiment(request, pipeline_id, pipeline_metadata,
                             output_path):
     """Create and launch a new Katib experiment.
@@ -191,11 +185,7 @@ def create_katib_experiment(request, pipeline_id, pipeline_metadata,
         yaml_file.write(yaml_text)
     _launch_katib_experiment(request, katib_experiment, namespace)
 
-    return {"name": katib_experiment["metadata"]["name"],
-            "namespace": namespace,
-            "status": None,
-            "trials": 0,
-            "maxTrialCount": katib_experiment["spec"]["maxTrialCount"]}
+    return _construct_experiment_return_base(katib_experiment, namespace)
 
 
 def get_experiment(request, experiment, namespace):
@@ -211,7 +201,7 @@ def get_experiment(request, experiment, namespace):
 
     Returns (dict): a dict describing the status of the running experiment
     """
-    k8s_co_client = _get_k8s_co_client(request.trans_id)
+    k8s_co_client = k8sutils.get_co_client()
 
     co_group = "kubeflow.org"
     co_version = "v1alpha3"
@@ -226,18 +216,21 @@ def get_experiment(request, experiment, namespace):
         raise RPCUnhandledError(message="Failed to get Katib experiment",
                                 details=str(e), trans_id=request.trans_id)
 
+    ret = _construct_experiment_return_base(exp, namespace)
+    if exp.get("status") is None:
+        return ret
+
     status, reason, message = _get_experiment_status(exp["status"])
-    return {"name": experiment,
-            "namespace": namespace,
-            "status": status,
-            "reason": reason,
-            "message": message,
-            "trials": exp["status"].get("trials", 0),
-            "trialsFailed": exp["status"].get("trialsFailed", 0),
-            "trialsRunning": exp["status"].get("trialsRunning", 0),
-            "trialsSucceeded": exp["status"].get("trialsSucceeded", 0),
-            "maxTrialCount": exp["spec"]["maxTrialCount"],
-            "currentOptimalTrial": exp["status"].get("currentOptimalTrial")}
+    ret.update({"status": status,
+                "reason": reason,
+                "message": message,
+                "trials": exp["status"].get("trials", 0),
+                "trialsFailed": exp["status"].get("trialsFailed", 0),
+                "trialsRunning": exp["status"].get("trialsRunning", 0),
+                "trialsSucceeded": exp["status"].get("trialsSucceeded", 0),
+                "currentOptimalTrial": exp["status"].get(
+                    "currentOptimalTrial")})
+    return ret
 
 
 def _get_experiment_status(experiment_status):
